@@ -8,19 +8,14 @@ import re
 import sys
 import types
 import unittest
-from nose.pyversion import ClassType, TypeType
+from nose.pyversion import ClassType, TypeType, isgenerator, ismethod
+
 
-try:
-    from compiler.consts import CO_GENERATOR
-except ImportError:
-    # IronPython doesn't have a complier module
-    CO_GENERATOR=0x20
-    
 log = logging.getLogger('nose')
 
 ident_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_.]*$')
 class_types = (ClassType, TypeType)
-skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)"
+skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)|(?:__pycache__)"
 
 try:
     set()
@@ -168,11 +163,6 @@ def isclass(obj):
     return obj_type in class_types or issubclass(obj_type, type)
 
 
-def isgenerator(func):
-    try:
-        return func.__code__.co_flags & CO_GENERATOR != 0
-    except AttributeError:
-        return False
 # backwards compat (issue #64)
 is_generator = isgenerator
 
@@ -270,7 +260,7 @@ def getpackage(filename):
     'nose.plugins'
     """
     src_file = src(filename)
-    if not src_file.endswith('.py') and not ispackage(src_file):
+    if (os.path.isdir(src_file) or not src_file.endswith('.py')) and not ispackage(src_file):
         return None
     base, ext = os.path.splitext(os.path.basename(src_file))
     if base == '__init__':
@@ -374,7 +364,7 @@ def split_test_name(test):
                 # nonsense like foo:bar:baz
                 raise ValueError("Test name '%s' could not be parsed. Please "
                                  "format test names as path:callable or "
-                                 "module:callable.")
+                                 "module:callable." % (test,))
     elif not tail:
         # this is a case like 'foo:bar/'
         # : must be part of the file path, so ignore it
@@ -440,7 +430,8 @@ def test_address(test):
             method_name = test._testMethodName
         return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], method_name))
-    if hasattr(test, '__class__') and test.__class__.__module__ != 'builtins':
+    if (hasattr(test, '__class__') and
+            test.__class__.__module__ not in ('__builtin__', 'builtins')):
         return test_address(test.__class__)
     raise TypeError("I don't know what %s is (%s)" % (test, t))
 test_address.__test__ = False # do not collect
@@ -456,9 +447,10 @@ def try_run(obj, names):
         if func is not None:
             if type(obj) == types.ModuleType:
                 # py.test compatibility
-                try:
-                    args, varargs, varkw, defaults = inspect.getargspec(func)
-                except TypeError:
+                if isinstance(func, types.FunctionType):
+                    args, varargs, varkw, defaults = \
+                        inspect.getargspec(func)
+                else:
                     # Not a function. If it's callable, call it anyway
                     if hasattr(func, '__call__'):
                         func = func.__call__
@@ -618,8 +610,13 @@ def transplant_func(func, module):
 
     """
     from nose.tools import make_decorator
-    def newfunc(*arg, **kw):
-        return func(*arg, **kw)
+    if isgenerator(func):
+        def newfunc(*arg, **kw):
+            for v in func(*arg, **kw):
+                yield v
+    else:
+        def newfunc(*arg, **kw):
+            return func(*arg, **kw)
 
     newfunc = make_decorator(func)(newfunc)
     newfunc.__module__ = module
